//Rainbow Acrylic Project - Feb 2021
//Rev2 - Changes to Mode funtionality
//Edwin Howard
//
//Uses a Adafruit Trinket with Neopixels (WS2812) to illuminate 7 acrylic semicircle arcs.
//Pinout:
//      PIN_0 : Neopixel data output
//      PIN_2 : Switch input, up position. Has 10k pull-up.
//      PIN_3 : Switch input, down position. Needs internal pull-up. Also USB pin, so may act strange while conneced to PC.
//
//LED Arrangement:
//      [19 , 18] 17 [16 , 15] 14 [13 , 12] 11 [10 , 9] 8 [7 , 6] 5 [4 , 3] 2 [1 , 0]
//         RED    x   ORANGE   x   YELLOW   x   GREEN   x  CYAN   x  BLUE   x  VIOLET
//
//Toggle Switch Function:
//     Switch Down: Fade through all colors at once
//     Switch Middle: Steady rainbow colors
//     Switch Up: Cycle rainbow colors across LED's.
//NOTE: The Left and Right sets of 20x Neopixels are mirrored, the data lines are split off together from Pin 0 (120ohm series resistors) and each side will mirror the other.
//NOTE: Code will use RNBW_LED_COUNT 0 - 6 and multiply by 3x to get pixel to turn on, then also turn on 3x + 1 for neighboring pixel.


#include <Adafruit_NeoPixel.h>

#define LED_DAT_PIN 0
#define BUTTON_UP 2
#define BUTTON_DN 3
#define LED_COUNT 20 // Total number of LED's.
#define RNBW_LED_COUNT 7

Adafruit_NeoPixel leds = Adafruit_NeoPixel(LED_COUNT, LED_DAT_PIN, NEO_GRB + NEO_KHZ800);
unsigned long getrainbowcolor(int);

void setup()
{
  leds.begin();  // Call this to start up the LED strip.
  leds.clear();   // Turns all LEDs off...
  leds.show();   // ...but the LEDs don't actually update until you call this.

  pinMode(BUTTON_UP,  INPUT); // Set push button input. Active low. Should already have physical 10k pull-up.
  pinMode(BUTTON_DN,  INPUT_PULLUP); // Set push button input. Active low. Enable internal 20k pull-up.
  
}

void loop() 
{

  leds.setBrightness(255);

  while(digitalRead(BUTTON_UP) == 1 && digitalRead(BUTTON_DN) == 1) // Steady rainbow colors!
  {
    leds.setPixelColor(6*3,0xFF0000); // RED
    leds.setPixelColor(6*3+1,0xFF0000);

    leds.setPixelColor(5*3,0xFF3F00); // ORANGE
    leds.setPixelColor(5*3+1,0xFF3F00);

    leds.setPixelColor(4*3,0xFFFF00); // YELLOW
    leds.setPixelColor(4*3+1,0xFFFF00);

    leds.setPixelColor(3*3,0x00FF00); // GREEN
    leds.setPixelColor(3*3+1,0x00FF00);

    leds.setPixelColor(2*3,0x00FFFF); // CYAN
    leds.setPixelColor(2*3+1,0x00FFFF);

    leds.setPixelColor(1*3,0x0000FF); // BLUE
    leds.setPixelColor(1*3+1,0x0000FF);

    leds.setPixelColor(0*3,0xFF00FF); // VIOLET
    leds.setPixelColor(0*3+1,0xFF00FF);
    
    leds.show();

    for(int i = 0; i<100;i++)
    {
        delay(100);
        if(digitalRead(BUTTON_UP) == 0 || digitalRead(BUTTON_DN) == 0)
            break;
    }
  } // End steady rainbow colors


  leds.clear();
  leds.show();
  delay(500);
  
  while(digitalRead(BUTTON_DN) == 0) // Fade through all colors at once
  {
    for(int j=0;j<384;j++)
    {
      for(int i=0;i<RNBW_LED_COUNT;i++)
      {
        leds.setPixelColor(i*3,getrainbowcolor(j));
        leds.setPixelColor(i*3+1,getrainbowcolor(j));
      }
      delay(20);
      leds.show();
      if(digitalRead(BUTTON_DN) == 1)
        break;
    }
  } // End fade through all colors

  leds.clear();
  leds.show();
  delay(500);

  while(digitalRead(BUTTON_UP) == 0) // Fade through colors across rainbow pattern
  {
    for(int j=0;j<384;j++)
    {
      leds.setPixelColor(6*3,getrainbowcolor(j)); // Outermost Arc, Start with Red
      leds.setPixelColor(6*3+1,getrainbowcolor(j));
    
      leds.setPixelColor(5*3,getrainbowcolor((j+16) % 384)); // Start with Orange
      leds.setPixelColor(5*3+1,getrainbowcolor((j+16) % 384));
     
      leds.setPixelColor(4*3,getrainbowcolor((j+64) % 384)); // Start with Yellow
      leds.setPixelColor(4*3+1,getrainbowcolor((j+64) % 384));
    
      leds.setPixelColor(3*3,getrainbowcolor((j+128) % 384)); // Start with Green
      leds.setPixelColor(3*3+1,getrainbowcolor((j+128) % 384));
    
      leds.setPixelColor(2*3,getrainbowcolor((j+192) % 384)); // Start with Cyan
      leds.setPixelColor(2*3+1,getrainbowcolor((j+192) % 384));
    
      leds.setPixelColor(1*3,getrainbowcolor((j+256) % 384)); // Start with Blue
      leds.setPixelColor(1*3+1,getrainbowcolor((j+256) % 384));
    
      leds.setPixelColor(0*3,getrainbowcolor((j+320) % 384)); // Innermost Arc, Start with Violet
      leds.setPixelColor(0*3+1,getrainbowcolor((j+320) % 384));

      delay(20);
      leds.show();
      if(digitalRead(BUTTON_UP) == 1)
        break;
    }
  } // Fade through colors across rainbow pattern

  leds.clear();
  leds.show();
  delay(500);

}

unsigned long getrainbowcolor(int a)//returns a hex color 0xRRGGBB from 0 to 383 on the rainbow scale
{
  if(a < 64)
    return 0xFF0000 + 0x000400*long(a);//from red to yellow : adding green
  if(a < 128)
    return 0xFFFF00 - 0x040000*long(a-64);//from yellow to green : subtracting red
  if(a < 192)
    return 0x00FF00 + 0x000004*long(a-128);//from green to cyan : adding blue
  if(a < 256)
    return 0x00FFFF - 0x000400*long(a-192);//from cyan to blue : subtracting green
  if(a < 320)
    return 0x0000FF + 0x040000*long(a-256);//from blue to violet : adding red
  if(a < 384)
    return 0xFF00FF - 0x000004*long(a-320);//from violet to red : subtracting blue

  return 0x000000; //reutrn off in case requested # is out of range
}
